/*
 *         	  OMEGA IRC SECURITY SERVICES
 * 	      	    (C) 2008 Omega Dev Team
 *
 *   See file AUTHORS in IRC package for additional names of
 *   the programmers.
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 1, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 *    $Id: access.c 1552 2009-12-14 06:57:11Z twitch $
 */


#include "stdinc.h"
#include "server.h"


/*******************************************/
/**
 * find_access() - Find the FIRST matching access
 * access list entry for the given user and return
 * the access list entry.
 *
 * @param u - Pointer to the user struct
 *            that we are checking with.
 *
 * @return Access* - return a pointer to the
 *                   access entry list that the user
 *                   that the user belongs to, or
 *                   NULL if no match is found.
 *
 */


Access* find_access(User *u)
{
	dlink_node *dl,*hdl;
	char *host;
	Access *a;
	char host_pattern[150];
	char ip_pattern[150];
	
    sprintf(host_pattern,"%s@%s",(u->user)? u->user : "*",u->host);
	sprintf(ip_pattern,"%s@%s",(u->user)? u->user : "*",u->ip);
	
	DLINK_FOREACH(dl,accesslist.head)
	{
		a = dl->data;

		DLINK_FOREACH(hdl,a->hosts.head)
		{
		 	host = hdl->data;

			if (match_wild(host,host_pattern,0))
				return a;
			if (match_wild(host,ip_pattern,0))
				return a;
			continue;
		}
	}
	return NULL;
}


/*******************************************/
/**
 * Check a users access against the specified level.
 *
 * @param u A pointer to the user structure which
 *          we are checking against.
 * @param level The level to check for.
 *
 * @return Return true if the access match, or return
 *         0 if we fail for any other reason.
 */


int check_access(User* u, char *level)
{

	if (!u)
		return 0;

	if (!level)
		return 1;

	//user has no access
	if (!u->access)
		return 0;

	if (strchr(u->access,*level))
		return 1;

	return 0;


}

/*******************************************/
/**
 * setAccess() - finds then sets or removes
 * access for a user, this will set the first
 * matching access level on a user, as provided '
 * by find_access().
 *
 *  @param u - The user struct we are working with
 *
 *
 *  @return int - Returns true if access has been set
 *                or false if no record was found.
 */


int setAccess(User* u)
{

	Access *a;

	if (!u)
		return 0;

	memset(u->access,0,sizeof(u->access));
	
	if (u->oper)
	{
		if (!(a = find_access(u)))
		{
			strlcpy(u->access,ACC_FLAG_OPER,sizeof(u->access));
			return 0;
		}
        strlcpy(u->access,a->flags,sizeof(u->access));
		return 1;
	}

	return 1;
}

/*************************************************/
/**
 *   Destroy all entries within the access list.
 *
 *   @note This destroyes all access list entries
 *         with that said, it shouldnt be called
 *         in any other circumstances besides rehash
 *         and shutdown, as it leaves us with no entries.
 *
 *
 */


int DestAccList()
{
	dlink_node *dl;
	dlink_node *tdl;

	dlink_node *hdl,*thdl; //host dlink list
	char *host;

	Access *a;

	DLINK_FOREACH_SAFE(dl,tdl,accesslist.head)
	{

		a = dl->data;

		DLINK_FOREACH_SAFE(hdl,thdl,a->hosts.head)
		{
			host = hdl->data;

			dlink_find_delete(host,&a->hosts);
			dlink_free(hdl);
			free(host);
		}


		dlink_find_delete(a,&accesslist);
		dlink_free(dl);
		free(a);

	}
	return 0;

}

/*************************************************/
/**
 *   refresh_access() - Crawls our user list and checks
 *   access on each entries host.
 *
 *   @note This method can be slower on larger nets and
 *   is designed only to update access on special instances
 *   such as rehash.
 *
 *  @XXX Crawling this big of a list is rediculous... i think we should keep a master list somewere
 * -Twitch
 */

void refresh_access(void)
{
    User* u = NULL;
    Access* a = NULL;

    dlink_node *udl = NULL;


     DLINK_FOREACH(udl,userlist.head)
     {
                u = udl->data;

		/**
		 * if there not an OPER or they are a SERVICE
         * they have no access. so skip that entry :)
         * this should give us back some speed and
         * proformance here. Esp on networks with more
         * Services and Users then IRCops (Which should
         * be all of them imo)
         * - Twitch
		 */

		if ((!u->oper) || (u->service))
			continue;

		setAccess(u);
     }

    return;
}
/*************************************************/

int add_access_flag (char *name,char *flag)
{
		dlink_node *dl;
		AccessFlag *af;
		
		if ((af = find_flag_byname(name)))
				return -1;
		if ((af = find_flag_byvalue(flag)))
				return -1;
				
		af = (AccessFlag*) malloc(sizeof(AccessFlag));
		if (!af)
				return 0;
		
		dl = dlink_create();
		
		strlcpy(af->name,name,sizeof(af->name));
		strlcpy(af->flag,flag,sizeof(af->flag));
		dlink_add_tail (af, dl, &access_flags);
		return 1;
}

/*************************************************/

void init_access_flags(void)
{
		add_access_flag("REHASH",ACC_FLAG_REHASH);
		add_access_flag("ACCESS",ACC_FLAG_ACCESS);
		add_access_flag("LIST",ACC_FLAG_LIST);
		add_access_flag("JOIN",ACC_FLAG_JOIN);
		add_access_flag("SET",ACC_FLAG_SET);
		add_access_flag("DIE",ACC_FLAG_DIE);
		add_access_flag("DEBUG",ACC_FLAG_DEBUG);
		add_access_flag("MODULE",ACC_FLAG_MODULE);
		
		add_access_flag("ROOT",ACC_ROOT);
		add_access_flag("SPOPER",ACC_STAFF);
		add_access_flag("STAFF",ACC_STAFF);
		add_access_flag("ADMIN",ACC_ADMN);
		add_access_flag("OPER", ACC_OPER);
		
}

/***********************************************/

AccessFlag *find_flag_byname(char *name)
{
		dlink_node *dl;
		AccessFlag *af;
		
		if (!name)
			return NULL;
		
		DLINK_FOREACH(dl,access_flags.head)
			{
				af = dl->data;
				
				if (strcasecmp(name,af->name)==0)
					return af;
			}
		return NULL;
}

/***********************************************/

AccessFlag *find_flag_byvalue(char* flag)
{
		dlink_node *dl;
		AccessFlag *af;
		
		if (!flag)
			return NULL;
		
		DLINK_FOREACH(dl,access_flags.head)
			{
				af = dl->data;
				
				if (*af->flag == *flag)
					return af;
			}
		return NULL;
}



/***********************************************/

int del_acces_flag(char *name)
{
		dlink_node *dl;
		AccessFlag *af;
		
		if (!name)
			return 0;
			
		if ((af = find_flag_byname(name)))
		{
			dl  = dlink_find_delete (af, &userlist);
			dlink_free (dl);
			free(af);
			return 1;
		}
		
		return 0;
}

/* EOF */

